"""Build strategies for different connector types.

The connector-builder-mcp supports multiple build strategies for different connector types:

- `connector_builder_mcp.build_strategies.declarative_yaml_v1` - Declarative YAML Source Connectors
- `connector_builder_mcp.build_strategies.declarative_openapi_v3` - Declarative Interfaces Built as Extensions on OpenAPI v3
- `connector_builder_mcp.build_strategies.kotlin_destination` - Kotlin destination connectors
- `connector_builder_mcp.build_strategies.kotlin_source` - Kotlin source connectors
"""
